/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/



/******************************************************************************
 * Includes
 *****************************************************************************/
#include "type.h"
#include "LPC11xx.h"
#include "capsense.h"
#include "adc.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

#define ADC_DONE           0x80000000
#define DEBOUNCE_VAL       (4)

/******************************************************************************
 * External global variables
 *****************************************************************************/



/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint16_t average[4];
static uint8_t state[4];
static uint8_t avgindex[4];
static uint8_t debounce[4];
static uint8_t initialized = FALSE;


/******************************************************************************
 * Local Functions
 *****************************************************************************/
static short ADC_ReadCH0(void)
{
    LPC_IOCON->R_PIO0_11 = 2;                      // AD0/P0.11 = AD0 ~~~~~~~~~ set sensor as AD0 input
    LPC_ADC->CR = (1<<0) |                         // select channel 0 on ADC
                 (3 << 8) |                        // ADC_CLK
                 (1 << 24);                        // start conversion

    while (!(LPC_ADC->DR[0] & ADC_DONE)) ;         // wait until end of AD1 conversion

    LPC_IOCON->R_PIO0_11 = 0x81;                   // AD0/P0.11 = GPO ~~~~~~~~~ charge the cap plate
    LPC_ADC->CR &= 0xF8FFFFFF;                     // stop ADC

    return (LPC_ADC->DR[0] >> 6) & 0x3FF;          // return A/D conversion value
}


static short ADC_ReadCH1(void)
{
    LPC_IOCON->R_PIO1_0 = 2;                       // AD1/P1.0 = AD1 ~~~~~~~~~ set sensor as AD1 input
    LPC_ADC->CR = (1<<1) |                         // select channel 1 on ADC
                 (3 << 8) |                        // ADC_CLK
                 (1 << 24);                        // start conversion

    while (!(LPC_ADC->DR[1] & ADC_DONE)) ;         // wait until end of AD1 conversion

    LPC_IOCON->R_PIO1_0 = 0x81;                    // AD1/P1.0 = GPO ~~~~~~~~~ charge the cap plate
    LPC_ADC->CR &= 0xF8FFFFFF;                     // stop ADC

    return (LPC_ADC->DR[1] >> 6) & 0x3FF;          // return A/D conversion value
}

static short ADC_ReadCH2(void)
{
    LPC_IOCON->R_PIO1_1 = 2;                       // AD2/P1.1 = AD2 ~~~~~~~~~ set sensor as AD2 input
    LPC_ADC->CR = (1<<2) |                         // select channel 2 on ADC
                 (3 << 8) |                        // ADC_CLK
                 (1 << 24);                        // start conversion

    while (!(LPC_ADC->DR[2] & ADC_DONE)) ;         // wait until end of AD1 conversion

    LPC_IOCON->R_PIO1_1 = 0x81;                    // AD2/P1.1 = GPO ~~~~~~~~~ charge the cap plate
    LPC_ADC->CR &= 0xF8FFFFFF;                     // stop ADC

    return (LPC_ADC->DR[2] >> 6) & 0x3FF;          // return A/D conversion value
}

static short ADC_ReadCH3(void)
{

    LPC_IOCON->R_PIO1_2 = 2;                       // AD2/P1.1 = AD2 ~~~~~~~~~ set sensor as AD2 input
    LPC_ADC->CR = (1<<3) |                         // select channel 2 on ADC
                 (3 << 8) |                        // ADC_CLK
                 (1 << 24);                        // start conversion

    while (!(LPC_ADC->DR[3] & ADC_DONE)) ;         // wait until end of AD1 conversion

    LPC_IOCON->R_PIO1_2 = 0x81;                    // AD2/P1.1 = GPO ~~~~~~~~~ charge the cap plate
    LPC_ADC->CR &= 0xF8FFFFFF;                     // stop ADC

    return (LPC_ADC->DR[3] >> 6) & 0x3FF;          // return A/D conversion value
}

static short ADC_ReadCH(int ch) {
	switch (ch) {
	case 0:
		return ADC_ReadCH0();
	case 1:
		return ADC_ReadCH1();
	case 2:
		return ADC_ReadCH2();
	case 3:
		return ADC_ReadCH3();
	}

	return 0;
}

/******************************************************************************
 * Public Functions
 *****************************************************************************/


/******************************************************************************
 *
 * Description:
 *    Initialize the Capacitive sensor Driver
 *
 *****************************************************************************/
void capsense_init (void)
{
	int i = 0;

	for (i = 0; i < 4; i++) {
		average[i] = 0;
		state[i] = 0;
		avgindex[i] = 0;
		debounce[i] = 0;
	}

	// Disable Power down bit to the ADC block.
	LPC_SYSCON->PDRUNCFG &= ~(0x1<<4);

	// Enable AHB clock to the ADC.
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<13);

	// AD0 -> p0.11
	// AD1 -> p1.0
	// AD2 -> p1.1
	// AD3 -> p1.2


	// set to output
	LPC_GPIO0->DIR |= (1<<11);
	LPC_GPIO1->DIR |= (1<<0);
	LPC_GPIO1->DIR |= (1<<1);
	LPC_GPIO1->DIR |= (1<<2);

	// warm up, establish average
    for (i=0; i<200; i++)                          // warm up, establish average
    {
        average[0] = (32 + ADC_ReadCH0() + (15 * average[0])) / 16;
        average[1] = (32 + ADC_ReadCH1() + (15 * average[1])) / 16;
        average[2] = (32 + ADC_ReadCH2() + (15 * average[2])) / 16;
        average[3] = (32 + ADC_ReadCH3() + (15 * average[3])) / 16;
    }

    initialized = TRUE;
}

/******************************************************************************
 *
 * Description:
 *    Get state of the capacitive sensors
 *
 * Returns:
 *   state of the capacitive sensors
 *
 *****************************************************************************/
uint8_t capsense_read(void)
{
	return ( state[3] << 3 | state[2] << 2 | state[1] << 1 | state[0] << 0);
}

void capsense_timer(void)
{
	int i = 0;
	uint16_t reading[4] = {0,0,0,0};
	char  result[4]   = {0,0,0,0};

	if (!initialized) {
		return;
	}

	for (i = 0; i < 4; i++) {

		reading[i] = ADC_ReadCH(i);

		// above (average + 6% of average) -> press condition
		if (reading[i] > average[i] + (average[i] >> 4))
		{
			if (debounce[i] == DEBOUNCE_VAL)
			{
				state[i] = 1;
				result[i] = 1;
			}
			else {
				debounce[i]++;
			}
		}

		// below (average + 3% of average) -> release condition
		else if (reading[i] < average[i] + (average[i] >> 5))
		{
			if (debounce[i] == 0)
			{
				state[i] = 0;
				result[i] = 0;
			}
			else {
				debounce[i]--;
			}
		}
/*
		// recalculate average when not pressed
		if (result[i] == 0 && debounce[i] == 0)
		{
			// average index delay
			if (++avgindex[i] == 8)
			{
				average[i] = (reading[i] + (15*average[i])) / 16;
				avgindex[i] = 0;
			}
		}
*/


	}
}

