/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
******************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "LPC11xx.h"
#include "lm75b.h"
#include "i2c.h"
#include "pcf8576.h"
#include "eeprom.h"
#include "adc.h"
#include "uart.h"
#include "capsense.h"


#include <cr_section_macros.h>
#include <NXP/crp.h>

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define TIME_INTERVAL (200)

#define BTN_SW1 (0x01)
#define BTN_SW2 (0x02)
#define BTN_SW4 (0x04)

#define LED2 (0x02)

#define STATE_SHOW_TRIM    (1)
#define STATE_SHOW_BUTTONS (2)
#define STATE_SHOW_TEMP    (3)
#define STATE_SHOW_EEPROM  (4)

/******************************************************************************
 * Global variables
 *****************************************************************************/

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP;

volatile uint32_t msTicks=0;

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint8_t ledToggle = 0;
static uint32_t ledLastToggle = 0;
static uint8_t state = 0;
static uint32_t adcVal = 0;

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void buttons_init(void)
{
	// SW1 -> GPIO, input
	LPC_IOCON->PIO0_3 &= ~0x07;
	LPC_GPIO0->DIR &= ~(1 << 3);

	// SW2 -> GPIO, input
	LPC_IOCON->PIO1_4 &= ~0x07;
	LPC_GPIO1->DIR &= ~(1 << 3);

	// SW4 -> GPIO, input
	LPC_IOCON->PIO0_1 &= ~0x07;
	LPC_GPIO0->DIR &= ~(1 << 1);
}

static uint8_t buttons_getState(void)
{
	uint8_t s = 0;
	uint32_t p0 = LPC_GPIO0->DATA;
	uint32_t p1 = LPC_GPIO1->DATA;

	if ((p0 & (1<<3)) == 0) {
		s |= BTN_SW1;
	}

	if ((p0 & (1<<1)) == 0) {
		s |= BTN_SW4;
	}

	if ((p1 & (1<<4)) == 0) {
		s |= BTN_SW2;
	}

	return s;
}

static void leds_init(void)
{
	// LED2 -> GPIO, output, off
	LPC_IOCON->PIO0_7 &= ~0x07;
	LPC_GPIO0->DIR |= 1 << 7;
	LPC_GPIO0->DATA  &= ~(1 << 7);

}

static void leds_setState(uint8_t ledOnMask)
{
	if ((ledOnMask & LED2) != 0) {
		LPC_GPIO0->DATA  |= 1 << 7;
	}
	else {
		LPC_GPIO0->DATA  &= ~(1 << 7);
	}

}

static void leds_toggle(void)
{
	if (msTicks > ledLastToggle + 1000) {

		if (ledToggle == 0) {
			ledToggle |= (LED2);
		}
		else {
			ledToggle = 0;
		}

		leds_setState(ledToggle);
		ledLastToggle = msTicks;
	}


}


/******************************************************************************
 * Public functions
 *****************************************************************************/

int main(void)
{
	uint32_t time=0;
	uint8_t eepromOk = 0;
	uint8_t btns = 0;

	int32_t tempValue;
	uint8_t cap = 0;

	SysTick_Config(SystemCoreClock / 1000);
	NVIC_SetPriority(SysTick_IRQn, 2);

	I2CInit( (uint32_t)I2CMASTER, 0 );
	NVIC_SetPriority(I2C_IRQn, 1);

	/* GPIO */
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);

    UARTInit(115200);

	ADCInit(9000000);

	LPC_IOCON->PIO0_7 &= ~0x07;
	LPC_GPIO0->DIR |= 1 << 7;
	LPC_GPIO0->DATA  &= ~(1 << 7);

	capsense_init();

	buttons_init();
	leds_init();

	UARTSendString((uint8_t*)"LPC11D14 QuickStart Board Demo\r\n");

	// initialize the LCD controller
	pcf8576_init();
	lcdWriteString((uint8_t*)"        THANK YOU FOR USING THE LPC11D14 QUICKSTART BOARD FROM EMBEDDED ARTISTS - BASED ON NXP'S AMAZING CORTEX-M0 LPC11D14        ", 1500, 350);

	// initialize the temperature sensor
	lm75b_config(LM75B_CFG_NORMAL);

	// check if the eeprom is OK
	eepromOk = eeprom_test();

	while (1) {

		cap = capsense_read();
		if (cap & CAP_0) {
			state = STATE_SHOW_TRIM;
		}
		else if (cap & CAP_1) {
			state = STATE_SHOW_BUTTONS;
		}
		else if (cap & CAP_2) {
			state = STATE_SHOW_TEMP;
		}
		else if (cap & CAP_3) {
			state = STATE_SHOW_EEPROM;
		}

		leds_toggle();

		switch(state) {
		case STATE_SHOW_TRIM:

			if (msTicks < time + 100) {
				break;
			}
			time = msTicks;

			lcdDisableScrolling();



			lcdWriteDigit('A', 0);
			lcdWriteDigit('D', 1);
			lcdWriteDigit('C', 2);
			lcdWriteDigit(':', 3);
			lcdWriteDigit('0' + (adcVal / 1000), 4);
			lcdWriteDigit('0' + ((adcVal / 100) % 10), 5);
			lcdWriteDigit('0' + ((adcVal / 10) % 10), 6);
			lcdWriteDigit('0' + (adcVal % 10), 7);
			lcdFlushDigits();

			break;
		case STATE_SHOW_BUTTONS:

			if (msTicks < time + 100) {
				break;
			}
			time = msTicks;

			lcdDisableScrolling();

			btns = buttons_getState();

			lcdWriteDigit('B', 0);
			lcdWriteDigit('T', 1);
			lcdWriteDigit('N', 2);
			lcdWriteDigit(':', 3);
			lcdWriteDigit(' ', 4);
			lcdWriteDigit(' ', 5);
			lcdWriteDigit(' ', 6);
			lcdWriteDigit(' ', 7);

			if ((btns & BTN_SW1) != 0) {
				lcdWriteDigit('S', 5);
				lcdWriteDigit('W', 6);
				lcdWriteDigit('1', 7);
			}


			if ((btns & BTN_SW2) != 0) {
				lcdWriteDigit('S', 5);
				lcdWriteDigit('W', 6);
				lcdWriteDigit('2', 7);
			}

			if ((btns & BTN_SW4) != 0) {
				lcdWriteDigit('S', 5);
				lcdWriteDigit('W', 6);
				lcdWriteDigit('4', 7);
			}

			lcdFlushDigits();

			break;
		case STATE_SHOW_TEMP:

			if (msTicks < time + 500) {
				break;
			}
			time = msTicks;


			tempValue = lm75b_readTemp() / 10;
			lcdDisableScrolling();

			lcdWriteDigit('T', 0);
			lcdWriteDigit(':', 1);
			lcdWriteDigit(' ', 2);
			lcdWriteDigit('0' + ((tempValue / 100) % 10), 3);
			lcdWriteDigit('0' + ((tempValue / 10) % 10), 4);
			lcdAddDpAfterDigit(4);
			lcdWriteDigit('0' + (tempValue % 10), 5);
			lcdWriteDigit('o', 6);
			lcdWriteDigit('C', 7);
			lcdFlushDigits();

			break;
		case STATE_SHOW_EEPROM:


			if (eepromOk) {
				lcdWriteString((uint8_t *)"E2P OK  ", 1500, 350);
			}
			else {
				lcdWriteString((uint8_t *)"E2P FAIL", 1500, 350);
			}

			break;
		}

	}
}

static int cnt = 0;
void SysTick_Handler(void)
{
	uint32_t restore = 0;

	msTicks++;
	lcdTimerHandler(1);
	capsense_timer();
	if (cnt++ >= 10) {

		// The capacitive sensors set the ADC clock to a high frequency
		// When using this high frequency when reading from the trimming
		// potentiometer we don't get accurate values
		//
		// Changing the ADC clock below only when reading the trimming
		// potentiometer and then chaning back again.

		restore = LPC_ADC->CR;
		LPC_ADC->CR = ((SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV)/9000000-1)<<8;

		adcVal = ADCRead(7);
		LPC_ADC->CR = restore;

	}

}
